// main.cpp

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "Pic.h"

void read_input (char[], char[]);
void read_data  (char[], char[], int, double[]);

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Main function for program Pic
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

int main ()
{
  Pic    pic;
  char   in_string[5];
  char*  cursor = (char*) "Pic> ";
  double data[5];
  
  // Main control loop
  printf ("\n");
  printf ("Program: Pic - Evolves electron equations of motion in unmagnetized, collisionless, 1D plasma\n");
  printf ("               with uniform background of singly-charged ions. Initial state consists of two\n");
  printf ("               warm counter-propagating electron beams.\n");
  printf ("Author : Richard Fitzpatrick, University of Texas at Austin\n");
  printf ("Date   : June 2017\n\n");
  printf ("Type `help` for help\n\n");
  while (1)
    {
      read_input (cursor, in_string);

      if ((in_string[0] == 'q') && (in_string[1] == 'u'))
	exit (0);
      else if ((in_string[0] == 'n') && (in_string[1] == 's'))
	{
	  read_data (cursor, (char*) "N ? ", 1, data);
	  if (!pic.SetN (int (data[0]))) 
	    printf ("%-sN = %6d\n", cursor, int (data[0]));
	} 
      else if ((in_string[0] == 'j') && (in_string[1] == 's'))
	{
	  read_data (cursor, (char*) "J ? ", 1, data);
	  if (!pic.SetJ (int (data[0]))) 
	    printf ("%-sJ = %6d\n", cursor, int (data[0]));
	} 
      else if ((in_string[0] == 'l') && (in_string[1] == 's'))
	{
	  read_data (cursor, (char*) "L ? ", 1, data);
	  if (!pic.SetL (data[0])) 
	    printf ("%-sL = %11.4e\n", cursor, data[0]);
	}
      else if ((in_string[0] == 'v') && (in_string[1] == 'b'))
	{
	  read_data (cursor, (char*) "vb ? ", 1, data);
	  if (!pic.Setvb (data[0])) 
	    printf ("%-svb = %11.4e\n", cursor, data[0]);
	}
      else if ((in_string[0] == 'd') && (in_string[1] == 't'))
	{
	  read_data (cursor, (char*) "dt ? ", 1, data);
	  if (!pic.Setdt (data[0])) 
	    printf ("%-sdt = %11.4e\n", cursor, data[0]);
	}
      else if ((in_string[0] == 't') && (in_string[1] == 'm'))
	{
	  read_data (cursor, (char*) "tmax ? ", 1, data);
	  if (!pic.Settmax (data[0])) 
	    printf ("%-stmax = %11.4e\n", cursor, data[0]);
	}
      else if ((in_string[0] == 's') && (in_string[1] == 't'))
	pic.Status ();	
      else if ((in_string[0] == 'h') && (in_string[1] == 'e'))
	{
	  printf ("\n");  
	  printf ("ns   ... set number of electrons, N\n");
	  printf ("js   ... set number of spatial grid points, J\n");
	  printf ("ls   ... set spatial domain length (in Debye lengths), L\n");
	  printf ("vb   ... set beam speed (in thermal speeds), vb\n");
	  printf ("dt   ... set time step (in inverse electron plasma frequencies), dt\n");
	  printf ("tmax ... set simulation time (in inverse electron plasma frequencies), tmax\n\n");
	  printf ("solv ... run simulation\n");
	  printf ("back ... run previous simulation backwards\n\n");
	  printf ("help ... print list of valid commands\n");
	  printf ("stat ... print current values of input parameters\n");
	  printf ("quit ... exit program\n\n");
	}
      else if ((in_string[0] == 's') && (in_string[1] == 'o'))
	{
	  pic.Solve ();
	}
      else if ((in_string[0] == 'b') && (in_string[1] == 'a'))
	{
	  pic.SolveBackwards ();
	}
    }
}

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Function to read line of input from terminal and store it in array input[] of length 5
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
void read_input (char cursor[], char input[])
{
  int count = 0; char c;

  printf ("%-s", cursor);
  while ((c = getchar ()) && (c != '\n'))
    {
      input[count] = c;
      if (count < 4) ++count;
    }
  input[count] = '\0';
}

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Function to read n data items from terminal and store them in double array x[]
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
void read_data (char cursor[], char message[], int n, double x[])
{
  int count = 1; char c;

  printf ("%-s%-s", cursor, message);
  for (count = 1; count <= n; count++)
    scanf ("%lf", &x[count - 1]);
  while ((c = getchar ()) && (c != '\n')) {}
}

  


